<?php

namespace Drupal\migrate_retry\EventSubscriber;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\migrate\Event\MigrateEvents;
use Drupal\migrate\Event\MigrateImportEvent;
use Drupal\migrate_retry\MigrateIdMapInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Shows during a migration that there are items needing retry.
 */
class MigrationSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * Logger, in case there are items needing retry.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * MigrationSubscriber constructor.
   *
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   Logger.
   */
  public function __construct(LoggerChannelInterface $logger) {
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      MigrateEvents::POST_IMPORT => 'onPostImport',
    ];
  }

  /**
   * Show items needing retry after executing a migration.
   *
   * @param \Drupal\migrate\Event\MigrateImportEvent $event
   *   The import event.
   */
  public function onPostImport(MigrateImportEvent $event) {
    $id_map = $event->getMigration()->getIdMap();
    $rows_needing_retry = $id_map->getRowsWithStatus(MigrateIdMapInterface::STATUS_NEEDS_RETRY, $id_map->processedCount());
    $rows_needing_retry_count = count($rows_needing_retry);
    if ($rows_needing_retry_count > 0) {
      $this->logger->warning($this->formatPlural($rows_needing_retry_count,
        'There is an item needing retry for this migration.', 'There are @numItems items needing retry for this migration.',
        [
          '@numItems' => $rows_needing_retry_count,
        ]
      ));
    }
  }

}
