<?php

namespace Drupal\migrate_retry\Plugin\QueueWorker;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\migrate\MigrateMessage;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate\Plugin\MigrationPluginManagerInterface;
use Drupal\migrate\Row;
use Drupal\migrate_retry\MigrateIdMapInterface;
use Drupal\migrate_retry\MigrationBusyException;
use Drupal\migrate_retry\RetryFailedException;
use Drupal\migrate_retry\MigrateExecutable;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines 'migrate_retry' queue worker.
 *
 * @QueueWorker(
 *   id = "migrate_retry",
 *   title = @Translation("MigrateRetry"),
 *   cron = {"time" = 60}
 * )
 */
class MigrateRetry extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * Machine name of the queue.
   *
   * @var string
   */
  const QUEUE_NAME = 'migrate_retry';

  /**
   * Allowed retries.
   *
   * @var int
   */
  const MAX_RETRIES = 5;

  /**
   * Queue, used to requeue in case of error.
   *
   * @var \Drupal\Core\Queue\QueueInterface
   */
  protected $queue;

  /**
   * Used to load the migrations so we do the retries.
   *
   * @var \Drupal\migrate\Plugin\MigrationPluginManagerInterface
   */
  protected $migrationPluginManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.migration'),
      $container->get('queue')
    );
  }

  /**
   * Constructs the migrate retry queue worker.
   *
   * @param array $configuration
   *   Plugin configuration.
   * @param string $plugin_id
   *   Plugin id.
   * @param array $plugin_definition
   *   Plugin definition.
   * @param \Drupal\migrate\Plugin\MigrationPluginManagerInterface $migrationPluginManager
   *   Loads any migration to migrate the elements that needs retry.
   * @param \Drupal\Core\Queue\QueueFactory $queueFactory
   *   Loads the migrate retry queue.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    array $plugin_definition,
    MigrationPluginManagerInterface $migrationPluginManager,
    QueueFactory $queueFactory) {
    $this->migrationPluginManager = $migrationPluginManager;
    $this->queue = $queueFactory->get(self::QUEUE_NAME);
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * Check the item will be processed is valid.
   *
   * @param mixed $data
   *   Data.
   *
   * @return bool
   *   TRUE when the retry information is available:
   *   - Source ids
   *   - Migration name.
   *   - Retries (it should be higher than 0).
   */
  protected function validItem($data) : bool {
    return is_object($data) && isset($data->migration_id)
      && isset($data->source_id_values)
      && is_array($data->source_id_values)
      && isset($data->retries) && is_int($data->retries) && $data->retries > 0;
  }

  /**
   * Handle an exception when the migration still fails.
   *
   * @param \stdClass $data
   *   Queued item.
   * @param \Drupal\migrate\Plugin\MigrationInterface $migration
   *   Migration.
   */
  protected function handleException(\stdClass $data, MigrationInterface $migration) {
    if ($data->retries > 0) {
      // @todo get the delay time from configuration!
      throw new RetryFailedException(1800, 'Element failed to import');
    }
    else {
      $row = $this->buildRow($data, $migration);
      $migration->getIdMap()->saveIdMapping($row, [], MigrateIdMapInterface::STATUS_FAILED);
    }
  }

  /**
   * Build migrate row for the queue item.
   *
   * @param \stdClass $data
   *   Queued item.
   * @param \Drupal\migrate\Plugin\MigrationInterface $migration
   *   Migration.
   *
   * @return \Drupal\migrate\Row
   *   Row built from source id values.
   */
  protected function buildRow(\stdClass $data, MigrationInterface $migration) : Row {
    $ids = $migration->getSourcePlugin()->getIds();
    $source_values = [];
    $i = 0;
    foreach ($ids as $key => $value) {
      $source_values[$key] = $data->source_id_values[$i];
      $i++;
    }
    return new Row($source_values, $source_values);
  }

  /**
   * Do the data import.
   *
   * @param \stdClass $data
   *   Data retrieved by queue.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\migrate\MigrateException
   */
  protected function doImport(\stdClass $data) {
    $migration = $this->migrationPluginManager->createInstance($data->migration_id);
    if ($migration instanceof MigrationInterface) {
      if ($migration->getStatus() == MigrationInterface::STATUS_IDLE) {
        $source_id_values = $data->source_id_values;
        $row = $this->buildRow($data, $migration);
        $source_row = $migration->getIdMap()->getRowBySource($row->getSourceIdValues());
        if (!empty($source_row) && $source_row['source_row_status'] == MigrateIdMapInterface::STATUS_IMPORTED) {
          return;
        }
        $id_list[] = $row->getSourceIdValues();
        $migration->getIdMap()->setUpdate($row->getSourceIdValues());
        $executable = new MigrateExecutable($migration, new MigrateMessage(), $id_list);
        $data->retries--;
        $executable->import();

        $row = $migration->getIdMap()->getRowBySource($source_id_values);
        if (!empty($row['source_row_status']) && $row['source_row_status'] !== MigrateIdMapInterface::STATUS_IMPORTED) {
          $this->handleException($data, $migration);
        }
      }
      else {
        // @todo get the delay time from configuration!
        throw new MigrationBusyException(1800, 'The migration with name ' . $migration->label() . ' is not idle.');
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    if ($this->validItem($data)) {
      $this->doImport($data);
    }
  }

}
