<?php

namespace Drupal\migrate_retry;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate\Plugin\MigrationPluginManagerInterface;
use Drupal\migrate\Plugin\RequirementsInterface;
use Drupal\migrate_retry\Plugin\QueueWorker\MigrateRetry;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * QueueManager service.
 */
class QueueManager implements QueueManagerInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The queue factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The migration plugin manager.
   *
   * @var \Drupal\migrate\Plugin\MigrationPluginManagerInterface
   */
  protected $migrationPluginManager;

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a QueueManager object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   The queue factory.
   * @param \Drupal\migrate\Plugin\MigrationPluginManagerInterface $migration_plugin_manager
   *   The migration plugin manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(ConfigFactoryInterface $config_factory, QueueFactory $queue_factory, MigrationPluginManagerInterface $migration_plugin_manager, ModuleHandlerInterface $module_handler) {
    $this->configFactory = $config_factory;
    $this->queueFactory = $queue_factory;
    $this->migrationPluginManager = $migration_plugin_manager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * Get the migrations that are set up for retrying.
   *
   * @return \Drupal\migrate\Plugin\MigrationInterface[]
   *   Migrations.
   */
  protected function getMigrations() : array {
    $migrations = [];
    $migration_ids = $this->configFactory->get('migrate_retry.settings')->get('migrations');
    foreach ($migration_ids as $migration_id) {
      $migration = $this->migrationPluginManager->createInstance($migration_id);
      try {
        if ($migration instanceof MigrationInterface && $migration->getSourcePlugin() instanceof RequirementsInterface) {
          $migration->getSourcePlugin()->checkRequirements();
        }
      }
      catch (\Exception $e) {
        // Migration is not properly set up, excluding.
        watchdog_exception('error', $e);
        continue;
      }
      $migrations[] = $migration;
    }
    return $migrations;
  }

  /**
   * {@inheritdoc}
   */
  public function canEnqueue() {
    return $this->queueFactory->get(MigrateRetry::QUEUE_NAME)->numberOfItems() === 0;
  }

  /**
   * {@inheritdoc}
   */
  public function enqueueRowsNeedingRetry() {
    foreach ($this->getMigrations() as $migration) {
      $rows_needing_retry = $migration->getIdMap()->getRowsWithStatus(MigrateIdMapInterface::STATUS_NEEDS_RETRY, 50);
      foreach ($rows_needing_retry as $row_needing_retry) {
        $source_id_values = [];
        foreach (get_object_vars($row_needing_retry) as $key => $value) {
          if (strpos($key, 'sourceid') === 0) {
            $source_id_values[] = $value;
          }
        }
        $this->enqueueMigrateRow($migration->id(), $source_id_values);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function enqueueMigrateRow($migration_id, array $source_id_values) {
    $this->moduleHandler->invokeAll('migrate_retry_enqueue_migrate_row', [$migration_id, $source_id_values]);
    $data = [
      'migration_id' => $migration_id,
      'source_id_values' => array_values($source_id_values),
      'retries' => MigrateRetry::MAX_RETRIES,
    ];
    $this->queueFactory->get(MigrateRetry::QUEUE_NAME)->createItem((object) $data);
  }

}
